# ################################################################# #
#### LOAD LIBRARY AND DEFINE CORE SETTINGS                       ####
# ################################################################# #

### Initialise
rm(list = ls())
library(apollo)
apollo_initialise()

### Set core controls
apollo_control = list(
  modelName  ="MNLDest",
  modelDescr ="Simple MNL model on mode choice RP data",
  indivID    ="indId"
)

# ################################################################# #
#### GENERATE SYNTHETIC DATA                                     ####
# ################################################################# #

nZon <- 20
nInd <- 200
nObs <- 10
set.seed(1)

### Travel time and cost between zones
tt <- matrix(exp(rnorm(nZon^2, mean=1.5, sd=2)),
            nrow=nZon, ncol=nZon)
tc <- matrix(exp(rnorm(nZon^2, mean=0.5, sd=2)),
            nrow=nZon, ncol=nZon)

### Zone characteristics
xz <- data.frame( x1=-log(runif(nZon))/1, x2=-log(runif(nZon))/1 )

### Socio-demographics
database <- data.frame(indId = rep(1:nInd, each=nObs),
                       hiInc = rep(runif(nInd)<=0.33, each=nObs), 
                       young = rep(runif(nInd)<=0.20, each=nObs), 
                       origi = sample(1:nZon, nInd*nObs, replace=TRUE),
                       desti = 0)

### Simulate choices
uti <- matrix(0, nrow=nInd*nObs, ncol=nZon)
btt <- -1.0 - 0.5*database$hiInc + 0.3*database$young
btc <- -0.5 + 0.2*database$hiInc - 0.2*database$young
for(z in 1:nZon) uti[,z] <- 2*xz$x1[z] - 1*xz$x2[z] + btt*tt[database$origi,z] + btc*tc[database$origi,z] + -log(-log(runif(nInd*nObs)))
database$desti <- apply(uti, MARGIN=1, which.max)
rm(uti, btc, btt, nInd, nObs, nZon, z)

# ################################################################# #
#### DEFINE MODEL PARAMETERS                                     ####
# ################################################################# #

### Vector of parameters: c(name1=value1, name2=value2, ...)
apollo_beta=c(b1  = 0, b2     = 0, 
              btt = 0, bttInc = 0, bttAge = 0, 
              btc = 0, btcInc = 0, btcAge = 0)

### Name of fixed parameters. "c()" for none.
apollo_fixed = c()

# ################################################################# #
#### GROUP AND VALIDATE INPUTS                                   ####
# ################################################################# #

apollo_inputs = apollo_validateInputs()
apollo_inputs[["tt"]] <- tt
apollo_inputs[["tc"]] <- tc
apollo_inputs[["xz"]] <- xz

# ################################################################# #
#### DEFINE MODEL AND LIKELIHOOD FUNCTION                        ####
# ################################################################# #

apollo_probabilities=function(apollo_beta, apollo_inputs, 
                              functionality="estimate"){

  ### Initialise
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  P = list()
  
  ### List of utilities
  Z  <- 20
  tt <- apollo_inputs[["tt"]]
  tc <- apollo_inputs[["tc"]]
  xz <- apollo_inputs[["xz"]]
  V = list()
  for(z in 1:Z) V[[paste0("d",z)]] = b1*xz$x1[z] + b2*xz$x2[z] + 
    (btt + bttInc*hiInc + bttAge*young)*tt[origi,z] + 
    (btc + btcInc*hiInc + btcAge*young)*tc[origi,z]
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives  = setNames(1:Z, paste0("d",1:Z)),
    avail         = 1, 
    choiceVar     = desti,
    V             = V,
    componentName = "MNL"
  )
  
  ### Compute probabilities using MNL model
  P[['model']] = apollo_mnl(mnl_settings, functionality)

  ### Process and return probabilities
  P = apollo_panelProd(P, apollo_inputs, functionality)
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# ################################################################# #
#### MODEL ESTIMATION AND RESULTS REPORT                         ####
# ################################################################# #
s <- list(writeIter=FALSE)
model = apollo_estimate(apollo_beta, apollo_fixed, 
                        apollo_probabilities, apollo_inputs,
                        estimate_settings=s)

apollo_modelOutput(model)

s <- list(saveEst=FALSE, saveCov=FALSE, saveCorr=FALSE,
          saveModelObject=FALSE)
apollo_saveOutput(model, saveOutput_settings=s)


